/* XPM */
static char *back_xpm[] = {
/* columns rows colors chars-per-pixel */
"16 16 29 1",
"  c black",
". c #2D2D2D",
"X c #3B5035",
"o c #3C5235",
"O c #415639",
"+ c #4A6342",
"@ c #5B7950",
"# c #678C63",
"$ c #698267",
"% c #7EAB78",
"& c #A9C4A7",
"* c #AAC5A8",
"= c #AAC7A8",
"- c #ADC6AB",
"; c #AEC7AC",
": c #ADC8AB",
"> c #B0C7AE",
", c #B0C9B0",
"< c #B1C9B0",
"1 c #B2C9B0",
"2 c #B3CAB1",
"3 c #B4CBB2",
"4 c #B5CEB5",
"5 c #B9CEB7",
"6 c #BBD0B9",
"7 c #BAD1BA",
"8 c #CDDECB",
"9 c #CFDFCC",
"0 c None",
/* pixels */
"0000000000000000",
"0000000 00000000",
"000000  00000000",
"00000 9 00000000",
"0000 82       00",
"000 83447462% 00",
"00 8=22--=;=$ 00",
"0.9:>--=---:# 00",
"00 X@@@@@@@@+ 00",
"000 X@@XXXXXO 00",
"0000 o@       00",
"00000 X 00000000",
"000000  00000000",
"0000000 00000000",
"0000000000000000",
"0000000000000000"
};

/* XPM */
static char *forward_xpm[] = {
/* columns rows colors chars-per-pixel */
"16 16 35 1",
"  c black",
". c #0F1308",
"X c #445B2C",
"o c #43543D",
"O c #4D6146",
"+ c #506B46",
"@ c #5B7950",
"# c #65815C",
"$ c #6C8562",
"% c #6B9060",
"& c #709867",
"* c #7F9F76",
"= c #77A16E",
"- c #8DB389",
"; c #A6BFA0",
": c #A4C3A2",
"> c #A5C4A3",
", c #A8C6A5",
"< c #A9C7A6",
"1 c #AAC6A7",
"2 c #A8C7A8",
"3 c #AAC7A8",
"4 c #ABC7A8",
"5 c #ABC7AA",
"6 c #AEC5A8",
"7 c #ACC8A9",
"8 c #ADC8AD",
"9 c #AEC8AD",
"0 c #AFC8AD",
"q c #B2CBB0",
"w c #BDD3B8",
"e c #BFD4BB",
"r c #C1D6BD",
"t c #C2D7BE",
"y c None",
/* pixels */
"yyyyyyyyyyyyyyyy",
"yyyyyyyy yyyyyyy",
"yyyyyyyy  yyyyyy",
"yyyyyyyy = yyyyy",
"yy       4& yyyy",
"yy rwetttq3&.yyy",
"yy 695,,,93>* yy",
"yy ;5,,14340>% y",
"yy $@@@@@@@@X yy",
"yy #+++++@@X yyy",
"yy       @X yyyy",
"yyyyyyyy X yyyyy",
"yyyyyyyy  yyyyyy",
"yyyyyyyy yyyyyyy",
"yyyyyyyyyyyyyyyy",
"yyyyyyyyyyyyyyyy"
};

/* XPM */
static char *reload_xpm[] = {
/* columns rows colors chars-per-pixel */
"16 16 16 1",
"  c black",
". c #35431B",
"X c #50652A",
"o c #6B7957",
"O c #728E3A",
"+ c #7C9B40",
"@ c #9BB46F",
"# c #9CB476",
"$ c #9FA498",
"% c #A8AFA0",
"& c #B1C491",
"* c #BAD09D",
"= c #BDD2A1",
"- c #CCDCB8",
"; c #DEE6D6",
": c None",
/* pixels */
"::::: ::::::::::",
":::::  :::::::::",
"::   %  ::::::::",
":  ;-=*  :::::::",
": $@++O  :::::::",
": &X .  ::::::::",
": # :  :::   :::",
": o : :::::   ::",
":   ::::: : ; ::",
"::   :::  : + ::",
":::::::  ; X+ ::",
"::::::  ;+++X ::",
"::::::  #+XX  ::",
":::::::  #   :::",
"::::::::  ::::::",
"::::::::: ::::::"
};

/* XPM */
static char *home_xpm[] = {
/* columns rows colors chars-per-pixel */
"16 16 65 1",
"  c black",
". c #0B0E09",
"X c #111111",
"o c gray8",
"O c #191919",
"+ c #3D1B14",
"@ c gray18",
"# c #273122",
"$ c #333D2F",
"% c #323232",
"& c #343434",
"* c #393939",
"= c #3F3F3F",
"- c #354031",
"; c #364530",
": c #431D15",
"> c #50231A",
", c #53251B",
"< c #57261D",
"1 c #612B20",
"2 c #763E2F",
"3 c #424A40",
"4 c gray29",
"5 c #465541",
"6 c #495545",
"7 c #5B5B5B",
"8 c #566252",
"9 c #586455",
"0 c #5B6657",
"q c #606060",
"w c #656565",
"e c gray40",
"r c #A54937",
"t c #A95849",
"y c gray52",
"u c #8B8B8B",
"i c #979797",
"p c #9B9B9B",
"a c #AAAAAA",
"s c #B4B4B4",
"d c gray71",
"f c #B6B6B6",
"g c #BBBBBB",
"h c #BCBCBC",
"j c gray77",
"k c gray79",
"l c #CACACA",
"z c gray80",
"x c #CECECE",
"c c gray81",
"v c #D2D2D2",
"b c LightGray",
"n c gray86",
"m c gainsboro",
"M c #DFDFDF",
"N c gray88",
"B c #E2E2E2",
"V c #E4E4E4",
"C c gray91",
"Z c #E9E9E9",
"A c #EAEAEA",
"S c gray93",
"D c #EFEFEF",
"F c gray100",
"G c None",
/* pixels */
"GGGGGGGGGGGGGGGG",
"GGG   G  GGGGGGG",
"GGG t  w7 GGGGGG",
"GGG 2 7iAe GGGGG",
"GGG X*ulCjp GGGG",
"GGG 4pAjAlCy GGG",
"GG 4slAlClCxp GG",
"G O AlAxC@3& o G",
"GGG M><+D*F% GGG",
"GGG N,r,S@*@ GGG",
"GGG M,r,Vhmh GGG",
"GGG B,r<ndvs GGG",
"GGG n<<:xhbs GGG",
"G 33005;#.86-$6G",
"G3  66666 6666GG",
"GGGGGGGGGGGGGGGG"
};


/* XPM */
static char * stop_1_xpm[] = {
/* width height colors chars-per-color */
"16 16 5 1",
/* colors */
" 	s None	m none	c None",
".	c #808080",
"X	c red",
"o	s iconColor2	m white	c white",
"O	s iconColor1	m black	c black",
/* image */
"    ........    ",
"   .XXXXXXXX.   ",
"  .XXXXXXXXXX.  ",
" .XXXXXXXXXXXX. ",
".XXXXXXXXXXXXXX.",
".XXoooooXoXXooXO",
".XoXXXoXoXoXoXoO",
".XXoXXoXoXoXooXO",
".XXXoXoXoXoXoXXO",
".XooXXoXXoXXoXXO",
".XXXXXXXXXXXXXXO",
".XXXXXXXXXXXXXXO",
" .XXXXXXXXXXXXO.",
"  .XXXXXXXXXXO. ",
"   .XXXXXXXXO.  ",
"    OOOOOOOO.   "};

/* XPM */
static char * stop_2_xpm[] = {
/* width height colors chars-per-color */
"16 16 5 1",
/* colors */
" 	s None	m none	c None",
".	c #808080",
"X	c red",
"o	s iconColor2	m white	c white",
"O	s iconColor1	m black	c black",
/* image */
"    ........    ",
"   .XXXXXXXX.   ",
"  .XXXoXXoXXX.  ",
" .XXXXoXoXoXXX. ",
"OXXXXXXoXXoXXXX.",
"OXXXXXXXXXoXXXX.",
"OXXXXXoooooXXXX.",
"OXXXXXXXXXoXXXX.",
"OXXXXXXoooXXXXX.",
"OXXXXXoXXXoXXXX.",
"OXXXXXXoooXXXXX.",
"OXXXXXXXXXXXXXX.",
".OXXXXoooooXXX. ",
" .OXXXXXoXoXX.  ",
"  .OXXXXXoXX.   ",
"   .OOOOOOO.    "};

/* XPM */
static char * stop_3_xpm[] = {
/* width height colors chars-per-color */
"16 16 5 1",
/* colors */
" 	s None	m none	c None",
".	c #808080",
"X	s iconColor1	m black	c black",
"o	c red",
"O	s iconColor2	m white	c white",
/* image */
"   .XXXXXXXX    ",
"  .Xoooooooo.   ",
" .Xoooooooooo.  ",
".Xoooooooooooo. ",
"Xoooooooooooooo.",
"Xoooooooooooooo.",
"XooOooOooOooOOo.",
"XooOoOoOoOoOooo.",
"XoOOoOoOoOooOoo.",
"XOoOoOoOoOoooOo.",
"XoOOooOoOOOOOoo.",
".oooooooooooooo.",
" .oooooooooooo. ",
"  .oooooooooo.  ",
"   .oooooooo.   ",
"    ........    "};

/* XPM */
static char * stop_4_xpm[] = {
/* width height colors chars-per-color */
"16 16 5 1",
/* colors */
" 	s None	m none	c None",
".	c #808080",
"X	s iconColor1	m black	c black",
"o	c red",
"O	s iconColor2	m white	c white",
/* image */
"    .XXXXXXX.   ",
"   .ooOoooooX.  ",
"  .ooOoOoooooX. ",
" .oooOOOOOooooX.",
".ooooooooooooooX",
".oooooOOOooooooX",
".ooooOoooOoooooX",
".oooooOOOooooooX",
".ooooOoooooooooX",
".ooooOOOOOoooooX",
".ooooOoooooooooX",
".ooooOooOooooooX",
" .oooOoOoOoooo. ",
"  .oooOooOooo.  ",
"   .oooooooo.   ",
"    ........    "};
